
/*
*****************************************************************************
*
*                          Copyright (c) 2000 2004
*
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of source code must retain the above copyright notice, 
*     this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright notice, 
*     this list of conditions and the following disclaimer in the documentation
*     and/or other materials provided with the distribution.
*  - The names of its contributors may not be used to endorse or promote 
*     products derived from this software without specific prior written 
*     permission.
*   
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
*  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
*  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
*  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
*  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
*  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
*  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
*  POSSIBILITY OF SUCH DAMAGE.
*
******************************************************************************
* File          : RcReader.cpp
* Author        : Zadig
* Creation date : 15/08/2004
*.............................................................................
* Description :
******************************************************************************
*/

/* ########################################################################## */
/* ### #include                                                               */
#include <stdlib.h>
#include "RcReader.h"

/* ########################################################################## */
/* ### Variables                                                              */

/* ########################################################################## */
/* ### Functions                                                              */

/*
******************************************************************************
*             Copyright (c) 2000 2004
*
* Function      : RcReader_Thread
* Author        : Zadig
* Creation date : 15/08/2004
*.............................................................................
* Description   :
******************************************************************************
*/
int32 RcReader_Thread(void *pc_Reader)
{
   return(((RcReader*)pc_Reader)->Thread());
}


/*
******************************************************************************
*             Copyright (c) 2000 2004
*
* Function      : RcReader
* Author        : Zadig
* Creation date : 15/08/2004
*.............................................................................
* Description   :
******************************************************************************
*/
RcReader::RcReader(uint16 i_Channel, uint16 i_ChannelNb)
{
   /* init all variables */
   b_Quit = false;
   i_CurChannel = i_Channel;
   i_NbChannel = i_ChannelNb;
   BMessage *pc_ControlMessage;
   BInputDevice   *pc_BercDevice;

   /* create the port to communicate with BeRC */
   i_RcPort = create_port(5, "RcHeader");
   if( (i_RcPort == B_BAD_VALUE) || (i_RcPort == B_NO_MORE_PORTS) )
      return;

   /* register our port to the berc input server device */
   pc_BercDevice = find_input_device("Be Remote Control");
   if(pc_BercDevice == NULL)
      return;

   pc_ControlMessage = new BMessage(RC_REGISTER_CLIENT);
   pc_ControlMessage->AddString("berc:port", "RcHeader");
   pc_ControlMessage->AddInt32("berc:filter", RC_KEY_UNMAPPED | RC_KEY_MAPPED );
   pc_BercDevice->Control(RC_REGISTER_CLIENT, pc_ControlMessage);
   free(pc_ControlMessage);

   /* start the keycodes read thread */
   i_Thread = spawn_thread(RcReader_Thread, "RcReader", B_DISPLAY_PRIORITY, this );
   resume_thread(i_Thread);

   return ;
} 


/*
******************************************************************************
*             Copyright (c) 2000 2004
*
* Function      : ~RcReader
* Author        : Zadig
* Creation date : 15/08/2004
*.............................................................................
* Description   :
******************************************************************************
*/
RcReader::~RcReader(void)
{
   status_t i_ret = B_OK;
   
   b_Quit = true;
   close_port(i_RcPort);
   
   wait_for_thread(i_Thread, &i_ret );   
   return ;
} 

/*
******************************************************************************
*             Copyright (c) 2000 2004
*
* Function      : SetChannelNb
* Author        : Zadig
* Creation date : 23/10/2004
*.............................................................................
* Description   :
******************************************************************************
*/
void RcReader::SetChannelNb(uint16 i_ChannelNb)
{
   i_NbChannel = i_ChannelNb;
}

/*
******************************************************************************
*             Copyright (c) 2000 2004
*
* Function      : ChannelHasChanged
* Author        : Zadig
* Creation date : 15/08/2004
*.............................................................................
* Description   :
******************************************************************************
*/
bool RcReader::ChannelHasChanged(uint16 *pi_Channel)
{
   if(*pi_Channel != i_CurChannel)
   {
      *pi_Channel = i_CurChannel;
      return(true);
   }
   return(false);
} 


/*
******************************************************************************
*             Copyright (c) 2000 2004
*
* Function      : Thread
* Author        : Zadig
* Creation date : 15/08/2004
*.............................................................................
* Description   :
******************************************************************************
*/
int32 RcReader::Thread(void)
{
   char  i_Key;
   uint8  *p_Data;
   int32  i_MsgCode, i_KeyMsg;
   int32 i_Size;
   uint32  i_MsgSize;
   status_t i_ret;
   BMessage *pc_KeyMessage;

   while(!b_Quit )
   {
      /***************
       * read header *
       ***************/
      i_Size =  port_buffer_size(i_RcPort);
      if( (i_Size == B_BAD_PORT_ID) ||
          (i_Size == B_TIMED_OUT) ||
          (i_Size == B_WOULD_BLOCK) )
         
      {
         /* read error */
         continue;
      }
      if(i_Size != sizeof(uint32))
      {
         /* this is not a header, remove the message from the port */
         p_Data = (uint8*) malloc(i_Size);
         read_port(i_RcPort, &i_MsgCode, p_Data, i_Size);
         free(p_Data);         
         continue;
      }

      /* read the header */
      i_Size = read_port(i_RcPort, &i_MsgCode, &i_MsgSize, sizeof(uint32));
      if(i_MsgCode != RC_PORT_HEADER_CODE)
         continue;
   
      /****************
       * read message *
       ****************/
      i_Size =  port_buffer_size(i_RcPort);
      if( (i_Size == B_BAD_PORT_ID) || (i_Size == B_TIMED_OUT) ||
          (i_Size == B_WOULD_BLOCK) )         
      {
         /* read error */
         continue;
      }

      /* read the message */
      p_Data = (uint8*) malloc(i_Size);
      i_Size = read_port(i_RcPort, &i_MsgCode, p_Data, i_MsgSize);
      if(i_MsgCode != RC_PORT_MESSAGE_CODE)
      {
         free(p_Data);         
         continue;
      }
      
      /* unflatten and analyze it */
      pc_KeyMessage = new BMessage();
      pc_KeyMessage->Unflatten((const char*)p_Data);
      
      switch(pc_KeyMessage->what)
      {
         case B_KEY_DOWN:
            i_ret = pc_KeyMessage->FindInt8((const char*)"byte", (int8*)&i_Key);
            if(i_ret != B_OK)
               break;

            if( (i_Key >= '1') && (i_Key <= '9') ) /* channel key */
               i_CurChannel = i_Key - '0';
            break;

         case B_UNMAPPED_KEY_DOWN:
            i_ret = pc_KeyMessage->FindInt32((const char*)"key", (int32*)&i_KeyMsg);
            if(i_ret != B_OK)
               break;

            if(i_KeyMsg == RC_KEY_NEXT)
            {
               i_CurChannel = (i_CurChannel + 1) % (i_NbChannel+1);
               if(i_CurChannel == 0)
                  i_CurChannel = 1;
            }
            else if(i_KeyMsg == RC_KEY_PREV)
            {
               if(i_CurChannel == 1)
                  i_CurChannel = i_NbChannel;
               else
                  i_CurChannel -= 1;
            }
            break;
      }
      free(p_Data);         
   }
   return(B_OK);
} 
